/*
 * kylin-os-manager
 *
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef COLLECTION_H_
#define COLLECTION_H_

#include <QObject>

#include "watcher.h"
#include "crash_info.h"

namespace crash_collection
{

class Collection : public QObject
{
    Q_OBJECT

public:
    Collection();
    ~Collection();

    Watcher m_watcher;

Q_SIGNALS:
    void collected(QString path);

private Q_SLOTS:
    void collectionCrashInfo(pid_t pid);

private:
    enum Field { Unknow = 0, StrackTrace, PackageDepends };

    bool collectionByBash(pid_t pid);
    bool generateReport(pid_t pid);
    bool generate(QString reportFilePath, CrashInfo &crashInfo);
    bool analyzeInfoFile(QString infoFilePath, CrashInfo &crashInfo);
    bool analyzeCoreFile(QString reportFilePath, QString coreFilePath);

    QByteArray getFieldValue(QString field);
    void setCurrentField(Field field);
    void clearFieldJudge(void);

    Field m_currentField = Field::Unknow;
};

} // namespace crash_collection

#endif