/*
 * kylin-os-manager
 *
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <QCoreApplication>
#include <QDBusConnection>
#include <QDBusError>
#include <QDebug>

#include "crash_interface.h"

int main(int argc, char *argv[])
{
    QCoreApplication app(argc, argv);
    app.setOrganizationName("kylin os manager team");
    app.setApplicationName("kylin os manager daemon");

    QDBusConnection systemBus = QDBusConnection::systemBus();
    if (!systemBus.isConnected()) {
        qCritical() << "d-bus connection fail !";
        return -1;
    }

    if (!systemBus.registerService("com.kylin.os.manager.daemon")) {
        qCritical() << "d-bus register service fail ! d-bus error : " << systemBus.lastError();
        return -1;
    }

    /* 发布崩溃收集对象 */
    if (!systemBus.registerObject("/com/kylin/os/manager/daemon/crash", new crash_collection::CrashInterface,
                                  QDBusConnection::ExportAllSlots | QDBusConnection::ExportAllSignals)) {
        qCritical() << "d-bus register crash object fail ! d-bus error : " << systemBus.lastError();
        return -1;
    }

    return app.exec();
}